@@ -862,7 +862,6 @@ static void
 ges_clip_init (GESClip * self)
 {
   self->priv = ges_clip_get_instance_private (self);
-  /* FIXME, check why it was done this way _DURATION (self) = GST_SECOND; */
   self->priv->layer = NULL;
   self->priv->nb_effects = 0;
 }
@@ -1271,16 +1270,18 @@ ges_clip_set_top_effect_index (GESClip * clip, GESBaseEffect * effect,
   g_return_val_if_fail (GES_IS_CLIP (clip), FALSE);
 
   track_element = GES_TRACK_ELEMENT (effect);
+  if (G_UNLIKELY (GES_CLIP (GES_TIMELINE_ELEMENT_PARENT (track_element)) !=
+          clip))
+    return FALSE;
+
   current_prio = _PRIORITY (track_element);
 
   _get_priority_range (GES_CONTAINER (clip), &min_prio, &max_prio);
 
   newindex = newindex + min_prio;
   /*  We don't change the priority */
-  if (current_prio == newindex ||
-      (G_UNLIKELY (GES_CLIP (GES_TIMELINE_ELEMENT_PARENT (track_element)) !=
-              clip)))
-    return FALSE;
+  if (current_prio == newindex)
+    return TRUE;
 
   if (newindex > (clip->priv->nb_effects - 1 + min_prio)) {
     GST_DEBUG ("You are trying to make %p not a top effect", effect);
