@@ -30,6 +30,7 @@
 
 #include "ges-asset.h"
 #include "ges-base-xml-formatter.h"
+#include "ges-timeline-tree.h"
 
 G_BEGIN_DECLS
 
@@ -50,54 +51,62 @@ GST_DEBUG_CATEGORY_EXTERN (_ges_debug);
 #define _DURATION(obj) GES_TIMELINE_ELEMENT_DURATION (obj)
 #define _MAXDURATION(obj) GES_TIMELINE_ELEMENT_MAX_DURATION (obj)
 #define _PRIORITY(obj) GES_TIMELINE_ELEMENT_PRIORITY (obj)
+#ifndef _END
 #define _END(obj) (_START (obj) + _DURATION (obj))
+#endif
 #define _set_start0 ges_timeline_element_set_start
 #define _set_inpoint0 ges_timeline_element_set_inpoint
 #define _set_duration0 ges_timeline_element_set_duration
 #define _set_priority0 ges_timeline_element_set_priority
 
 #define GES_TIMELINE_ELEMENT_FORMAT \
-    "s:%p" \
-    " start: %" GST_TIME_FORMAT \
-    " inpoint: %" GST_TIME_FORMAT \
-    " duration: %" GST_TIME_FORMAT
+    "s<%p>" \
+    " [ %" GST_TIME_FORMAT \
+    " (%" GST_TIME_FORMAT \
+    ") - %" GST_TIME_FORMAT "(%" GST_TIME_FORMAT") layer: %" G_GINT32_FORMAT "] "
 
 #define GES_TIMELINE_ELEMENT_ARGS(element) \
     GES_TIMELINE_ELEMENT_NAME(element), element, \
     GST_TIME_ARGS(GES_TIMELINE_ELEMENT_START(element)), \
     GST_TIME_ARGS(GES_TIMELINE_ELEMENT_INPOINT(element)), \
-    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_DURATION(element))
+    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_DURATION(element)), \
+    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_MAX_DURATION(element)), \
+    GES_TIMELINE_ELEMENT_LAYER_PRIORITY(element)
+
+#define GES_FORMAT GES_TIMELINE_ELEMENT_FORMAT
+#define GES_ARGS GES_TIMELINE_ELEMENT_ARGS
 
 G_GNUC_INTERNAL gboolean
-timeline_ripple_object         (GESTimeline *timeline, GESTrackElement *obj,
-                                    GList * layers, GESEdge edge,
-                                    guint64 position);
+timeline_ripple_object         (GESTimeline *timeline, GESTimelineElement *obj,
+                                gint new_layer_priority,
+                                GList * layers, GESEdge edge,
+                                guint64 position);
 
 G_GNUC_INTERNAL gboolean
 timeline_slide_object          (GESTimeline *timeline, GESTrackElement *obj,
                                     GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_roll_object           (GESTimeline *timeline, GESTrackElement *obj,
-                                    GList * layers, GESEdge edge, guint64 position);
+timeline_roll_object           (GESTimeline *timeline, GESTimelineElement *obj,
+                                GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_trim_object           (GESTimeline *timeline, GESTrackElement * object,
-                                    GList * layers, GESEdge edge, guint64 position);
+timeline_trim_object           (GESTimeline *timeline, GESTimelineElement * object,
+                                guint32 new_layer_priority, GList * layers, GESEdge edge,
+                                guint64 position);
 G_GNUC_INTERNAL gboolean
 ges_timeline_trim_object_simple (GESTimeline * timeline, GESTimelineElement * obj,
-                                 GList * layers, GESEdge edge, guint64 position, gboolean snapping);
+                                 guint32 new_layer_priority, GList * layers, GESEdge edge,
+                                 guint64 position, gboolean snapping);
 
 G_GNUC_INTERNAL gboolean
 ges_timeline_move_object_simple (GESTimeline * timeline, GESTimelineElement * object,
                                  GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_move_object           (GESTimeline *timeline, GESTrackElement * object,
-                                    GList * layers, GESEdge edge, guint64 position);
-
-G_GNUC_INTERNAL gboolean
-timeline_context_to_layer      (GESTimeline *timeline, gint offset);
+timeline_move_object           (GESTimeline *timeline, GESTimelineElement * object,
+                                guint32 new_layer_priority, GList * layers, GESEdge edge,
+                                guint64 position);
 
 G_GNUC_INTERNAL void
 timeline_add_group             (GESTimeline *timeline,
@@ -121,6 +130,14 @@ gboolean
 timeline_remove_element       (GESTimeline *timeline,
                                GESTimelineElement *element);
 
+G_GNUC_INTERNAL
+GNode *
+timeline_get_tree           (GESTimeline *timeline);
+
+G_GNUC_INTERNAL
+void
+timeline_update_transition (GESTimeline *timeline);
+
 G_GNUC_INTERNAL
 void
 timeline_fill_gaps            (GESTimeline *timeline);
@@ -353,7 +370,6 @@ G_GNUC_INTERNAL void _ges_container_set_priority_offset   (GESContainer * contai
  ****************************************************/
 G_GNUC_INTERNAL void              ges_clip_set_layer              (GESClip *clip, GESLayer  *layer);
 G_GNUC_INTERNAL gboolean          ges_clip_is_moving_from_layer   (GESClip *clip);
-G_GNUC_INTERNAL guint32           ges_clip_get_layer_priority     (GESClip *clip);
 G_GNUC_INTERNAL void              ges_clip_set_moving_from_layer  (GESClip *clip, gboolean is_moving);
 G_GNUC_INTERNAL GESTrackElement*  ges_clip_create_track_element   (GESClip *clip, GESTrackType type);
 G_GNUC_INTERNAL GList*            ges_clip_create_track_elements  (GESClip *clip, GESTrackType type);
@@ -369,7 +385,6 @@ G_GNUC_INTERNAL void layer_set_priority               (GESLayer * layer, guint p
  ****************************************************/
 #define         NLE_OBJECT_TRACK_ELEMENT_QUARK                  (g_quark_from_string ("nle_object_track_element_quark"))
 G_GNUC_INTERNAL gboolean  ges_track_element_set_track           (GESTrackElement * object, GESTrack * track);
-G_GNUC_INTERNAL guint32   _ges_track_element_get_layer_priority (GESTrackElement * element);
 G_GNUC_INTERNAL void ges_track_element_copy_properties          (GESTimelineElement * element,
                                                                  GESTimelineElement * elementcopy);
 
@@ -396,8 +411,21 @@ G_GNUC_INTERNAL GESVideoTestSource * ges_video_test_source_new (void);
 /****************************************************
  *              GESTimelineElement                  *
  ****************************************************/
+typedef enum
+{
+  GES_CLIP_IS_MOVING = (1 << 0),
+  GES_TIMELINE_ELEMENT_SET_SIMPLE = (1 << 1),
+} GESTimelineElementFlags;
+
 G_GNUC_INTERNAL gdouble ges_timeline_element_get_media_duration_factor(GESTimelineElement *self);
 G_GNUC_INTERNAL GESTimelineElement * ges_timeline_element_get_copied_from (GESTimelineElement *self);
+G_GNUC_INTERNAL GESTimelineElementFlags ges_timeline_element_flags (GESTimelineElement *self);
+G_GNUC_INTERNAL void                ges_timeline_element_set_flags (GESTimelineElement *self, GESTimelineElementFlags flags);
+
+#define ELEMENT_FLAGS(obj)             (ges_timeline_element_flags (GES_TIMELINE_ELEMENT(obj)))
+#define ELEMENT_SET_FLAG(obj,flag)     (ges_timeline_element_set_flags(GES_TIMELINE_ELEMENT(obj), (ELEMENT_FLAGS(obj) | (flag))))
+#define ELEMENT_UNSET_FLAG(obj,flag)   (ges_timeline_element_set_flags(GES_TIMELINE_ELEMENT(obj), (ELEMENT_FLAGS(obj) & ~(flag))))
+#define ELEMENT_FLAG_IS_SET(obj,flag)  ((ELEMENT_FLAGS (obj) & (flag)) == (flag))
 
 /******************************
  *  GESMultiFile internal API *
